﻿using System;
using System.Collections.Generic;
using System.Text;

using Eco.Interfaces;
using Eco.ObjectRepresentation;
using Eco.Persistence;
using Eco.Services;


namespace Adecs.Eco.Helpers
{
    public class GeneralObjectRetrieval
    {
        private IEcoServiceProvider FEcoProvider;

        protected IEcoServiceProvider EcoProvider
        {
            get
            {
                return FEcoProvider;
            }
        }

        public GeneralObjectRetrieval(IEcoServiceProvider ecoProvider)
        {
            if (ecoProvider == null)
                throw new ArgumentNullException("ecoProvider");

            FEcoProvider = ecoProvider;
        }

        public IList<T> GetObjects<T>()
        {
            IOclPsService psService = EcoServiceHelper.GetOclPsService(this.EcoProvider);
            IOclService oclService = EcoServiceHelper.GetOclService(this.EcoProvider);

            string sTemp = typeof(T).ToString();
            string className = sTemp.Substring(sTemp.LastIndexOf(".") + 1);

            IObjectList result = psService.Execute(className + ".allInstances");
            result = oclService.Evaluate(className + ".allLoadedObjects") as IObjectList;
            return result.GetAsIList<T>();
        }

        public IList<T> GetObjectsMatchingCriteria<T>(string criteria)
        {
            if (criteria == null)
                throw new ArgumentNullException("criteria");

            IOclPsService psService = EcoServiceHelper.GetOclPsService(this.EcoProvider);
            IOclService oclService = EcoServiceHelper.GetOclService(this.EcoProvider);

            string sTemp = typeof(T).ToString();
            string className = sTemp.Substring(sTemp.LastIndexOf(".") + 1);

            IObjectList result = psService.Execute(className + ".allInstances" + criteria);
            result = oclService.Evaluate(className + ".allLoadedObjects" + criteria) as IObjectList;
            return result.GetAsIList<T>();
        }
        
        public IList<T> GetObjectsMatchingCriteria<T>(string criteria, IExternalVariableList variables)
        {
            if (variables == null)
                throw new ArgumentNullException("variables");
            if (criteria == null)
                throw new ArgumentNullException("criteria");

            IOclPsService psService = EcoServiceHelper.GetOclPsService(this.EcoProvider);
            IOclService oclService = EcoServiceHelper.GetOclService(this.EcoProvider);

            // Extract the class name from the type
            string sTemp = typeof(T).ToString();
            string className = sTemp.Substring(sTemp.LastIndexOf(".") + 1);

            string sOclPsExpression = className + ".allInstances" + criteria;
            string sOclExpression = className + ".allLoadedObjects" + criteria;
            IObjectList result = psService.Execute(null, variables, sOclPsExpression, -1, 0);
            result = oclService.Evaluate(sOclExpression, variables) as IObjectList;
            return result.GetAsIList<T>();
        }

        public IList<T> GetObjectsMatchingCriteria<T>(string criteria, string ordering)
        {
            if (criteria == null)
                throw new ArgumentNullException("criteria");
            if (ordering == null)
                throw new ArgumentNullException("ordering");

            IOclPsService psService = EcoServiceHelper.GetOclPsService(this.EcoProvider);
            IOclService oclService = EcoServiceHelper.GetOclService(this.EcoProvider);

            // Extract the class name from the type
            string sTemp = typeof(T).ToString();
            string className = sTemp.Substring(sTemp.LastIndexOf(".") + 1);
            
            // Get instances from the DB and already loaded
            string sOclPsExpression = className + ".allInstances" + criteria;
            string sOclExpression = className + ".allLoadedObjects" + criteria;
            IObjectList result = psService.Execute(sOclPsExpression, -1, 0);
            result = oclService.Evaluate(sOclExpression) as IObjectList;

            // Order the results
            string sOclOrdering = "self" + ordering;
            result = oclService.Evaluate(result, sOclOrdering) as IObjectList;

            return result.GetAsIList<T>();
        }

        public IElement ExecuteOcl(string OclExpression, IElement root)
        {
            if (string.IsNullOrEmpty(OclExpression))
                throw new ArgumentNullException("OclExpression");

            //IOclPsService ps = EcoServiceHelper.GetOclPsService(this.EcoProvider);
            IOclService ocl = EcoServiceHelper.GetOclService(this.EcoProvider);

            //IObjectList result = ps.Execute(root, OclExpression);
            return ocl.Evaluate(root, OclExpression);
        }
    }
}
